<?php
/* --------------------------------------------------------------
   GambioHubUpdaterInterface.inc.php 2017-09-29
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * HubConnectorUpdaterInterface
 */
interface HubConnectorUpdaterInterface
{
	/**
	 * HubConnectorUpdater constructor.
	 */
	public function __construct();
	
	
	/**
	 * Returns the current value of the "download before update" option.
	 *
	 * @return bool
	 */
	public function getDownloadBeforeUpdate();
	
	
	/**
	 * Returns the current version of the Hub Connector
	 *
	 * @return string
	 */
	public function getHubConnectorVersion();
	
	
	/**
	 * Returns the current state of the Hub Connector Updater
	 *
	 * @return string
	 */
	public function getHubConnectorUpdaterVersion();
	
	
	/**
	 * Returns the update status as an array. The array contains the keys body and info, which contains the body and header information of the curl request that was made.
	 *
	 * @return array
	 */
	public function getUpdatesStatus();
	
	
	/**
	 * Returns an arraywith all update versions which are already downloaded.
	 *
	 * @return array
	 */
	public function getDownloadedUpdates();
	
	
	/**
	 * Starts the download of all available updates.
	 *
	 * @throws \HubConnectorUpdaterDownloadFailedException
	 * @throws \HubConnectorUpdaterNoUpdateAvailableException
	 */
	public function downloadUpdates();
	
	
	/**
	 * Starts the installation of all available downloads. Updates will be downloaded bevore installation, if the "download before update" option is active.
	 *
	 * @throws \HubConnectorUpdaterDownloadFailedException
	 * @throws \HubConnectorUpdaterNoInitialPointException
	 * @throws \HubConnectorUpdaterNoUpdateAvailableException
	 * @throws \HubConnectorUpdaterUpdateFailedException
	 * @throws \HubConnectorUpdaterUpdatesNotDownloadedException
	 */
	public function installUpdates();
}