<?php

/* --------------------------------------------------------------
   HubClientKeySerializer.php 2017-01-25
   Gambio GmbH
   http://www.gambio.de
   Copyright © 2017 Gambio GmbH
   Released under the MIT License
   [https://opensource.org/licenses/MIT]
   --------------------------------------------------------------
*/
namespace HubPublic\Serializers;

use HubPublic\Exceptions\BadSerializerValueException;
use HubPublic\Serializers\Interfaces\SerializerInterface;
use HubPublic\ValueObjects\HubClientKey;
class HubClientKeySerializer implements SerializerInterface
{
    public function deserialize($string)
    {
        if (empty($string)) {
            throw new BadSerializerValueException('Given string is empty.');
        }
        $json = json_decode($string, true);
        // error for malformed json strings
        if ($json === null && json_last_error() > 0) {
            throw new BadSerializerValueException('Provided JSON string is malformed and could not be parsed: ' . $string);
        }
        return new HubClientKey($json['clientKey']);
    }
    public function serialize($value, $encode = true)
    {
        if (!is_object($value) || !$value instanceof HubClientKey) {
            throw new BadSerializerValueException('Argument is not a HubClientKey: ' . gettype($value));
        }
        $json = ['clientKey' => $value->asString()];
        return $encode ? json_encode($json, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES) : $json;
    }
}